@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -35,8 +35,8 @@ public:
   ~CXMLprefs() { UnloadXML(); }
 
   // Implementation
-  bool Load();
-  bool Store();
+  bool XML_Load();
+  bool XML_Store(const stringT &csBaseKeyName);
   bool Lock(stringT &locker); // if fails, locker points to culprit
   void Unlock();
 
@@ -63,7 +63,7 @@ public:
   // Remove a host/user from current configuration file
   bool RemoveHostnameUsername(const stringT &sHost, const stringT &sUser,
                               bool &bNoMoreNodes);
-  
+
   enum {XML_SUCCESS = 0,
         XML_LOAD_FAILED,
         XML_NODE_NOT_FOUND,
@@ -80,6 +80,6 @@ private:
   // CreateXML - bLoad will skip creation of root element
   bool CreateXML(bool bLoad);
   void UnloadXML();
-  stringT m_Reason; // why something bad happenned
+  stringT m_Reason; // why something bad happened
 };
 #endif /* __XMLPREFS_H */
