@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -14,6 +14,7 @@
 #include "AddEdit_PropertyPage.h"
 #include "ExtThread.h"
 #include "ControlExtns.h"
+#include "TBMStatic.h"
 
 #include "core/ItemData.h"
 
@@ -27,16 +28,15 @@ public:
 
   CAddEdit_Basic(CWnd *pParent, st_AE_master_data *pAEMD);
 
-  static CString CS_SHOW, CS_HIDE;
-  static CSecString HIDDEN_NOTES;
+  static CString CS_SHOW, CS_HIDE, CS_EXTERNAL_EDITOR, CS_HIDDEN_NOTES;
+  static HANDLE ghEvents[2];
 
-    // Dialog Data
+  // Dialog Data
   //{{AFX_DATA(CAddEdit_Basic)
   enum { IDD = IDD_ADDEDIT_BASIC, IDD_SHORT = IDD_ADDEDIT_BASIC_SHORT };
 
   CSecString m_password, m_password2;
-  CSecString m_notes;
-
+ 
   CComboBoxExtn m_ex_group;
 
   CEditExtn m_ex_title;
@@ -44,6 +44,7 @@ public:
   CRichEditExtn m_ex_notes;
   CEditExtn m_ex_URL;
   CEditExtn m_ex_email;
+  CEditExtn m_ex_base;
 
   CSecEditExtn m_ex_password, m_ex_password2;
 
@@ -54,9 +55,11 @@ public:
   CStaticExtn m_stc_notes;
   CStaticExtn m_stc_URL;
   CStaticExtn m_stc_email;
-  CStaticExtn m_stc_protected;
+  CStaticExtn m_stc_isdependent;
+  CStaticExtn m_stc_dependent;
 
-  CButton m_ViewDependentsBtn;
+  CComboBox m_cmbDependents;
+  CEditExtn m_ex_hidden_notes;
   //}}AFX_DATA
 
   CExtThread *m_thread; // worker thread
@@ -66,13 +69,21 @@ public:
   bool m_isPWHidden, m_isNotesHidden;
   bool m_bWordWrap, m_bLaunchPlus;
 
+  void CancelThreadWait()
+  { SetEvent(ghEvents[1]); }
+
+  bool IsNotesExternalEditorActive()
+  { return m_bUsingNotesExternalEditor; }
+
   // Overrides
   // ClassWizard generate virtual function overrides
-  //{{AFX_VIRTUAL(CAddEdit_Basic)
+
 protected:
-  BOOL PreTranslateMessage(MSG* pMsg);
+  BOOL PreTranslateMessage(MSG *pMsg);
+
+  //{{AFX_VIRTUAL(CAddEdit_Basic)
   virtual BOOL OnInitDialog();
-  virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
+  virtual void DoDataExchange(CDataExchange *pDX);    // DDX/DDV support
   virtual BOOL OnApply();
   virtual BOOL OnKillActive();
   //}}AFX_VIRTUAL
@@ -82,13 +93,12 @@ protected:
   //{{AFX_MSG(CAddEdit_Basic)
   afx_msg void OnHelp();
   afx_msg LRESULT OnQuerySiblings(WPARAM wParam, LPARAM);
+  afx_msg void OnPageKillActive(NMHDR *nmhdr, LRESULT *pLResult);
   afx_msg HBRUSH OnCtlColor(CDC *pDC, CWnd *pWnd, UINT nCtlColor);
 
-  afx_msg void OnPasskeySetFocus();
   afx_msg void OnENSetFocusPassword();
   afx_msg void OnENSetFocusPassword2();
   afx_msg void OnENChangePassword();
-  afx_msg void OnENSetFocusNotes();
   afx_msg void OnENKillFocusNotes();
   afx_msg void OnChanged();
   afx_msg void OnENChangeNotes();
@@ -97,9 +107,9 @@ protected:
   afx_msg void OnGroupComboChanged();
 
   afx_msg void OnGeneratePassword();
+  afx_msg void OnCopyPassword();
   afx_msg void OnShowPassword();
   afx_msg void OnSTCExClicked(UINT nId);
-  afx_msg void OnViewDependents();
   afx_msg void OnLaunch();
   afx_msg void OnSendEmail();
 
@@ -116,12 +126,20 @@ private:
   void SelectAllNotes();
   void ShowPassword();
   void HidePassword();
-  void ShowNotes();
-  void HideNotes();
+  void ShowNotes(const bool bForceShow = false);
+  void HideNotes(const bool bForceHide = false);
+  void ResetHiddenNotes();
+  void SetUpDependentsCombo();
+  void SetComboBoxWidth();
+
   bool CheckNewPassword(const StringX &group, const StringX &title,
                         const StringX &user, const StringX &password,
                         const bool bIsEdit, const CItemData::EntryType InputType, 
                         pws_os::CUUID &base_uuid, int &ibasedata, bool &b_msg_issued);
+  void SetGroupComboBoxWidth();
+  void ShowHideBaseInfo(const CItemData::EntryType &entrytype, CSecString &csBase);
+
+  CTBMStatic m_Help1, m_Help2, m_Help3, m_Help4;
 
   COLORREF m_group_cfOldColour, m_title_cfOldColour, m_user_cfOldColour;
   COLORREF m_pswd_cfOldColour, m_notes_cfOldColour, m_URL_cfOldColour;
@@ -129,7 +147,11 @@ private:
   BOOL m_bOKSave, m_bOKCancel;
 
   bool m_bInitdone;
+  bool m_bUsingNotesExternalEditor;
   int m_iPointSize;
+  int m_iLineCount;
+
+  CBitmap m_CopyPswdBitmap;
 };
 //-----------------------------------------------------------------------------
 // Local variables:
