@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -11,7 +11,7 @@
 //-----------------------------------------------------------------------------
 
 #include "PasswordSafe.h"
-#include "Windowsdefs.h"
+
 #include "ThisMfcApp.h"
 #include "GeneralMsgBox.h"
 #include "Shortcut.h"
@@ -228,8 +228,11 @@ int DboxMain::RestoreSafe()
     m_core.UnlockFile(m_core.GetCurFile().c_str());
   }
 
-  // clear the data before restoring
-  ClearData();
+  // Reset core and clear ALL associated data
+  m_core.ReInit();
+
+  // Clear application data
+  ClearAppData();
 
   // Validate it unless user says NO
   CReport Rpt;
@@ -408,27 +411,39 @@ void DboxMain::OnOptions()
     if (pOptionsPS->UpdateShortcuts()) {
       // Create vector of shortcuts for user's config file
       std::vector<st_prefShortcut> vShortcuts;
-      MapMenuShortcutsIter iter, iter_entry;
+      MapMenuShortcutsIter iter, iter_entry, iter_parent;
       m_MapMenuShortcuts = pOptionsPS->GetMaps();
 
       for (iter = m_MapMenuShortcuts.begin(); iter != m_MapMenuShortcuts.end();
            iter++) {
         // User should not have these sub-entries in their config file
         if (iter->first == ID_MENUITEM_GROUPENTER  ||
-            iter->first == ID_MENUITEM_VIEW        || 
+            iter->first == ID_MENUITEM_VIEWENTRY        || 
             iter->first == ID_MENUITEM_DELETEENTRY ||
             iter->first == ID_MENUITEM_DELETEGROUP ||
             iter->first == ID_MENUITEM_RENAMEENTRY ||
             iter->first == ID_MENUITEM_RENAMEGROUP) {
           continue;
         }
+
         // Now only those different from default
-        if (iter->second.siVirtKey  != iter->second.siDefVirtKey  ||
-            iter->second.cModifier != iter->second.cDefModifier) {
+        if (iter->second.siVirtKey != iter->second.siDefVirtKey ||
+          iter->second.cModifier != iter->second.cDefModifier) {
+          iter_parent = m_MapMenuShortcuts.find(iter->second.uiParentID);
+          std::wstring name(L"");
+          do {
+            name = iter_parent->second.name + std::wstring(L"->") + name;
+            iter_parent = m_MapMenuShortcuts.find(iter_parent->second.uiParentID);
+          } while (iter_parent != m_MapMenuShortcuts.end());
+
+          name += iter->second.name;
+          Remove(name, L'&');
+
           st_prefShortcut stxst;
           stxst.id = iter->first;
           stxst.siVirtKey = iter->second.siVirtKey;
           stxst.cModifier = iter->second.cModifier;
+          stxst.Menu_Name = name;
           vShortcuts.push_back(stxst);
         }
       }
@@ -446,8 +461,8 @@ void DboxMain::OnOptions()
 
       // Set up the shortcuts based on the main entry
       // for View, Delete and Rename
-      iter = m_MapMenuShortcuts.find(ID_MENUITEM_EDIT);
-      iter_entry = m_MapMenuShortcuts.find(ID_MENUITEM_VIEW);
+      iter = m_MapMenuShortcuts.find(ID_MENUITEM_EDITENTRY);
+      iter_entry = m_MapMenuShortcuts.find(ID_MENUITEM_VIEWENTRY);
       iter_entry->second.SetKeyFlags(iter->second);
 
       SetupSpecialShortcuts();
@@ -489,7 +504,15 @@ void DboxMain::OnOptions()
     // in this session with this database, until the database is closed.
     Command *pcmd;
 
-    if (m_core.GetReadFileVersion() == PWSfile::VCURRENT) {
+    // Save current horizontal scroll bar position in case view is refreshed
+    if (m_ctlItemList.GetItemCount() == 0) {
+      m_iListHBarPos = m_iTreeHBarPos = 0;
+    } else {
+      m_iListHBarPos = m_ctlItemList.GetScrollPos(SB_HORZ);
+      m_iTreeHBarPos = m_ctlItemTree.GetScrollPos(SB_HORZ);
+    }
+
+    if (m_core.GetReadFileVersion() >= PWSfile::V30) { // older versions don't have prefs
       if (sxOldDBPrefsString != sxNewDBPrefsString) {
         // Determine whether Tree needs redisplaying due to change
         // in what is shown (e.g. usernames/passwords)
@@ -548,7 +571,7 @@ void DboxMain::OnOptions()
     // If DB preferences changed and/or password history options
     if (pmulticmds != NULL) {
       int num_altered(0);
-      if (pmulticmds->GetSize() > 0) {
+      if (!pmulticmds->IsEmpty()) {
         // Do it
         Execute(pmulticmds);
 
@@ -589,7 +612,11 @@ void DboxMain::OnOptions()
           if (num_altered > 0) {
             ChangeOkUpdate();
           }
-        } 
+        }
+
+        // Restore current horizontal scroll bar position
+        m_ctlItemList.Scroll(CSize(m_iListHBarPos, 0));
+        m_ctlItemTree.SetScrollPos(SB_HORZ, m_iTreeHBarPos);
       } else {
         // Was created but no commands added in the end.
         delete pmulticmds;
@@ -715,7 +742,7 @@ void DboxMain::OnManagePasswordPolicies()
       StringX sxNewDBPrefsString(PWSprefs::GetInstance()->Store(true));
 
       // Set up Command to update string in database
-      if (m_core.GetReadFileVersion() == PWSfile::VCURRENT) {
+      if (m_core.GetReadFileVersion() >= PWSfile::V30) { // older versions don't have prefs
         Command *pcmd_undo = DBPrefsCommand::Create(&m_core, sxNewDBPrefsString);
         pmulticmds->Add(pcmd_undo);
       }
