@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -87,7 +87,7 @@ CPasswordPolicyDlg::CPasswordPolicyDlg(UINT uicaller, CWnd *pParent, bool bLongP
     m_Symbols = m_oldSymbols = m_st_default_pp.symbols.c_str();
   else {
     // empty, set to appropriate default
-    stringT st_symbols;
+    std::wstring st_symbols;
     if (m_PWEasyVision == TRUE)
       st_symbols = CPasswordCharPool::GetEasyVisionSymbols();
     else if (m_PWMakePronounceable == TRUE)
@@ -293,16 +293,20 @@ BOOL CPasswordPolicyDlg::OnInitDialog()
       // Load bitmap
       UINT nImageID = PWSprefs::GetInstance()->GetPref(PWSprefs::UseNewToolbar) ?
         IDB_COPYPASSWORD_NEW : IDB_COPYPASSWORD_CLASSIC;
-      BOOL brc = m_CopyPswdBitmap.Attach(::LoadImage(
-                                                     ::AfxFindResourceHandle(MAKEINTRESOURCE(nImageID), RT_BITMAP),
-                                                     MAKEINTRESOURCE(nImageID), IMAGE_BITMAP, 0, 0,
-                                                     (LR_DEFAULTSIZE | LR_CREATEDIBSECTION | LR_SHARED)));
+      BOOL brc = m_CopyPswdBitmap.Attach(
+                    ::LoadImage(::AfxFindResourceHandle(MAKEINTRESOURCE(nImageID), RT_BITMAP),
+                    MAKEINTRESOURCE(nImageID), IMAGE_BITMAP, 0, 0,
+                    (LR_DEFAULTSIZE | LR_CREATEDIBSECTION | LR_SHARED)));
+
       ASSERT(brc);
 
-      FixBitmapBackground(m_CopyPswdBitmap);
-      CButton *pBtn = (CButton *)GetDlgItem(IDC_COPYPASSWORD);
-      pBtn->SetBitmap(m_CopyPswdBitmap);
-
+      if (brc) {
+        FixBitmapBackground(m_CopyPswdBitmap);
+        CButton *pBtn = (CButton *)GetDlgItem(IDC_COPYPASSWORD);
+        ASSERT(pBtn != NULL);
+        if (pBtn != NULL)
+          pBtn->SetBitmap(m_CopyPswdBitmap);
+      }
       break;
     }
   case IDS_PSWDPOLICY:
@@ -392,10 +396,7 @@ BOOL CPasswordPolicyDlg::OnInitDialog()
     int iTime = m_pToolTipCtrl->GetDelayTime(TTDT_AUTOPOP);
     m_pToolTipCtrl->SetDelayTime(TTDT_AUTOPOP, 4 * iTime);
 
-    // Set the tooltip text
-    CString cs_ToolTip;
-    cs_ToolTip.LoadString(IDS_CLICKTOCOPY);
-    m_pToolTipCtrl->AddTool(GetDlgItem(IDC_COPYPASSWORD), cs_ToolTip);
+    AddTool(IDC_COPYPASSWORD, IDS_CLICKTOCOPY);
   }
 
   // Set appropriate focus
@@ -934,7 +935,7 @@ void CPasswordPolicyDlg::OnENOwnSymbols()
   CString cs_symbols;
   m_SymbolsEdit.GetWindowText(cs_symbols);
 
-  // Check if user about to leave own symbols emtpy
+  // Check if user about to leave own symbols empty
   if (cs_symbols.GetLength() == 0) {
     // Tell user
     CGeneralMsgBox gmb;
@@ -1095,7 +1096,6 @@ void CPasswordPolicyDlg::UnselectNamedPolicy()
   m_cbxPolicyNames.EnableWindow(FALSE);
 }
 
-
 void CPasswordPolicyDlg::OnSymbolReset()
 {
   do_reset_symbols(true);
@@ -1103,7 +1103,7 @@ void CPasswordPolicyDlg::OnSymbolReset()
 
 void CPasswordPolicyDlg::do_reset_symbols(bool restore_defaults)
 {
-  stringT st_symbols;
+  std::wstring st_symbols;
   if (m_PWEasyVision == TRUE)
     st_symbols = CPasswordCharPool::GetEasyVisionSymbols();
   else if (m_PWMakePronounceable == TRUE)
