@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -17,7 +17,6 @@
 
 #include "PasswordSafe.h"
 
-#include "Windowsdefs.h"
 #include "resource.h"
 #include "resource2.h"  // Menu, Toolbar & Accelerator resources
 #include "resource3.h"  // String resources
@@ -46,11 +45,13 @@
 #include "os/file.h"
 #include "os/env.h"
 #include "os/lib.h"
+#include "os/debug.h"
 
 #include "Shlwapi.h"
 
 #include <vector>
 #include <errno.h>
+#include <fstream>
 #include <io.h>
 
 using namespace std;
@@ -179,7 +180,8 @@ ThisMfcApp::~ThisMfcApp()
 static void Usage()
 {
   CGeneralMsgBox gmb;
-  gmb.AfxMessageBox(IDS_USAGE);
+  CString csMsg(MAKEINTRESOURCE(IDS_USAGE)), csTitle(MAKEINTRESOURCE(IDS_INVALIDCOMMANDLINE));
+  gmb.AfxMessageBox(csMsg, csTitle, MB_OK);
 }
 
 // tests if file exists, returns true if so, displays error message if not
@@ -255,7 +257,7 @@ static void GetVersionInfoFromFile(const CString &csFileName,
   // Get version information from the given file
   dwVerInfoSize = ::GetFileVersionInfoSize((LPWSTR)(LPCWSTR)csFileName, &dwVerHnd);
   if (dwVerInfoSize > 0) {
-    char* pVersionInfo = new char[dwVerInfoSize];
+    char *pVersionInfo = new char[dwVerInfoSize];
     if (pVersionInfo != NULL) {
       BOOL bRet = ::GetFileVersionInfo((LPWSTR)(LPCWSTR)csFileName,
                     (DWORD)dwVerHnd, (DWORD)dwVerInfoSize, (LPVOID)pVersionInfo);
@@ -308,14 +310,14 @@ void ThisMfcApp::LoadLocalizedStuff()
   const CString cs_HelpDir(PWSdirs::GetHelpDir().c_str());
   const CString cs_ExePath(PWSdirs::GetExeDir().c_str());
   
-  // See if user overridden default via Environmental Variable
+  // See if user overridden default via Environment Variable
   bool bPLRC(FALSE);
   
   // See if user has set preference instead.
   StringX sxLL = PWSprefs::GetInstance()->GetPref(PWSprefs::LanguageFile);
 
   if (!sxLL.empty()) {
-    // Preferences trumps Environmental Variable!!!!!
+    // Preferences trumps Environment Variable!!!!!
     size_t len = sxLL.length();
     ASSERT(len == 2 || len == 5);
     if (len == 2) {
@@ -371,17 +373,24 @@ void ThisMfcApp::LoadLocalizedStuff()
   cs_ResPath.Format(format_string, static_cast<LPCWSTR>(cs_LANG),
                        static_cast<LPCWSTR>(cs_CTRY));
   m_hInstResDLL = HMODULE(pws_os::LoadLibrary(LPCTSTR(cs_ResPath),
-                                              pws_os::LOAD_LIBRARY_APP));
+                                              pws_os::LOAD_LIBRARY_RESOURCE));
 
   if (m_hInstResDLL == NULL && !cs_CTRY.IsEmpty()) {
     // Now try base
     cs_ResPath.Format(L"pwsafe%s.dll", static_cast<LPCWSTR>(cs_LANG));
     m_hInstResDLL = HMODULE(pws_os::LoadLibrary(LPCTSTR(cs_ResPath),
-                                                pws_os::LOAD_LIBRARY_APP));
+                                                pws_os::LOAD_LIBRARY_RESOURCE));
   }
 
   if (m_hInstResDLL == NULL) {
     pws_os::Trace(L"Could not load language DLLs - using embedded resources.\n");
+    // If the requested DLL is not default English, show an error so we know the specifics.
+    // At a minimum this will show the actual error code.
+    if (!(cs_LANG == L"EN" && cs_CTRY.IsEmpty())) {
+		  CString errMessage;
+		  errMessage.Format(L"Attempt to load %s", LPCTSTR(cs_ResPath));
+		  pws_os::IssueError(LPCTSTR(errMessage));
+	  }
   } else { // successfully loaded a resource dll, check version
     DWORD dw_fileMajorMinor, dw_fileBuildRevision;
     GetVersionInfoFromFile(cs_ResPath, dw_fileMajorMinor, dw_fileBuildRevision);
@@ -494,9 +503,9 @@ void ThisMfcApp::SetupMenu()
   pMenu1 = m_pMainMenu->GetSubMenu(pos1);
   minfo.dwMenuData = ID_FILEMENU;
   
-  if (pMenu1 != NULL) { // Look for "Close Database"
+  if (pMenu1 != NULL) { // Look for "Lock Database"
     pMenu1->SetMenuInfo(&minfo);
-    pos2 = FindMenuItem(pMenu1, ID_MENUITEM_CLOSE);
+    pos2 = FindMenuItem(pMenu1, ID_MENUITEM_LOCK);
   }
   else
     pos2 = -1;
@@ -590,6 +599,14 @@ void ThisMfcApp::SetupMenu()
   minfo.dwMenuData = ID_VIEWMENU;
   pMenu1->SetMenuInfo(&minfo);
 
+  // Do View Menu Subview submenu
+  pos2 = app.FindMenuItem(pMenu1, ID_SUBVIEWMENU);
+  ASSERT(pos2 != -1);
+
+  pMenu2 = pMenu1->GetSubMenu(pos2);
+  minfo.dwMenuData = ID_SUBVIEWMENU;
+  pMenu2->SetMenuInfo(&minfo);
+
   // Do View Menu Filter submenu
   pos2 = app.FindMenuItem(pMenu1, ID_FILTERMENU);
   ASSERT(pos2 != -1);
@@ -661,8 +678,8 @@ bool ThisMfcApp::GetConfigFromCommandLine(StringX &sxConfigFile, StringX &sxHost
 {
   // ONLY get config file, host & user name if supplied
   // This MUST be done before PWSprefs is first initialised
-  bool rc(false);
   sxConfigFile = sxHost = sxUser = L"";
+  wstring sConfig;
 
   if (m_lpCmdLine[0] != L'\0') {
     CString args = m_lpCmdLine;
@@ -690,41 +707,117 @@ bool ThisMfcApp::GetConfigFromCommandLine(StringX &sxConfigFile, StringX &sxHost
           // ensure there's another non-flag argument
           if ((arg + 1) == argvec.end() || (arg + 1)[0] == L'-') {
             // Better stop as we don't know how bad things are!
+            // However - tell user about issue!
+            Usage();
             return false;
           } else {
             arg++;
             sxUser = *arg;
-            rc = true;
           }
           break;
         case L'H': case L'h': // set effective host
           // ensure there's another non-flag argument
           if ((arg + 1) == argvec.end() || (arg + 1)[0] == L'-') {
             // Better stop as we don't know how bad things are!
+            // However - tell user about issue!
+            Usage();
             return false;
           } else {
             arg++;
             sxHost = *arg;
-            rc = true;
           }
           break;
         case L'G': case L'g': // override default config file
           // ensure there's another non-flag argument
           if ((arg + 1) == argvec.end() || (arg + 1)[0] == L'-') {
             // Better stop as we don't know how bad things are!
+            // However - tell user about issue!
+            Usage();
             return false;
           } else {
             arg++;
-            if (!pws_os::FileExists(std::wstring(*arg))) {
-              // FileExists returns false if specified file not found
-              // DON'T use PWSprefs here
-              // Better stop as we don't know how bad things are!
-              return false;
+            bool bRelative(false);
+            if (PathIsRelative(*arg)) {
+              // As per Help entry - use executable directory unless overridden
+              // Don't use PWSdirs::GetConfigDir()
+              wstring sPWS_PREFSDIR = pws_os::getenv("PWS_PREFSDIR", true);
+              if (sPWS_PREFSDIR.empty()) {
+                sConfig = pws_os::getexecdir() + wstring(*arg);
+              } else {
+                sConfig = sPWS_PREFSDIR + wstring(*arg);
+              }
+              bRelative = true;
             } else {
-              sxConfigFile = *arg;
-              rc = true;
+              sConfig = *arg;
+            }
+
+            // Now resolve and tidy up absolute path
+            wchar_t full[_MAX_PATH + 1];
+            if (_wfullpath(full, sConfig.c_str(), _MAX_PATH) != NULL) {
+              sConfig = full;
+            }
+
+            if (!pws_os::FileExists(sConfig)) {
+              // FileExists returns false if specified file not found
+              CString cs_ErrorMsg, csConfigfile;
+              // If the user hasn't specified a language for issueing error messages
+              // during command line processing and before the configuration file
+              // has been opened, this message will be in English (as will any display
+              // of the Usage.
+              if (bRelative) {
+                // Also tell user the full path we checked as well.
+                csConfigfile.Format(L"%s\n  (%s)", static_cast<LPCWSTR>(*arg), 
+                  static_cast<LPCWSTR>(sConfig.c_str()));
+              } else {
+                // Just tell the user the absolute path they specified
+                csConfigfile = *arg;
+              }
+
+              CGeneralMsgBox gmb;
+              cs_ErrorMsg.Format(L"Configuration file not found.\n  %s\n\nDo you wish to create it?",
+                static_cast<LPCWSTR>(csConfigfile));
+
+              INT_PTR rc = gmb.AfxMessageBox(cs_ErrorMsg, L"Error finding configuration file",
+                                 MB_YESNO | MB_ICONINFORMATION | MB_DEFBUTTON2);
+
+              if (rc == IDNO) {
+                // Better stop as we don't know how bad things are!
+                return false;
+              }
+
+              // Try to create it
+              FILE *configfile = pws_os::FOpen(sConfig.c_str(), _T("w"));
+              if (configfile == NULL) {
+                // Can't create it either!!!
+                // Give detailed error message, if possible
+                DWORD error = GetLastError();
+                LPTSTR lpMsgBuf = NULL;
+                DWORD dw = FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
+                                  NULL,
+                                  error,
+                                  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+                                  (LPTSTR)&lpMsgBuf,
+                                  0, NULL);
+                if (dw != 0) {
+                  CGeneralMsgBox gmb_err;
+                  cs_ErrorMsg.Format(L"Unable to create configuration file:\n\n\t%s", lpMsgBuf);
+
+                  gmb_err.AfxMessageBox(cs_ErrorMsg, L"Error",
+                    MB_OK | MB_ICONINFORMATION);
+
+                  // Free buffer
+                  LocalFree(lpMsgBuf);
+                }
+                // Better stop as we don't know how bad things are!
+                return false;
+              }
+
+              // Close & delete it & drop through
+              fclose(configfile);
+              pws_os::DeleteAFile(sConfig);
             }
           }
+          sxConfigFile = sConfig.c_str();
           break;
         } // switch on flag
       }
@@ -732,7 +825,7 @@ bool ThisMfcApp::GetConfigFromCommandLine(StringX &sxConfigFile, StringX &sxHost
     } // while argvec
   } // Command line not empty
 
-  return rc;
+  return true;
 }
 
 bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
@@ -891,50 +984,10 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
           // Obsolete - databases are always validated during opening unless --novalidate specified
           break;
         case L'U': case L'u': // set effective user
-          // ensure there's another non-flag argument
-          if ((arg + 1) == argvec.end() || (arg + 1)[0] == L'-') {
-            Usage();
-            return false;
-          } else {
-            arg++;
-            // No need to set effective user as done in previous parsing
-            // of command line to get config information
-          }
-          break;
         case L'H': case L'h': // set effective host
-          // ensure there's another non-flag argument
-          if ((arg + 1) == argvec.end() || (arg + 1)[0] == L'-') {
-            Usage();
-            return false;
-          } else {
-            arg++;
-            // No need to set effective host as done in previous parsing
-            // of command line to get config information
-          }
-          break;
         case L'G': case L'g': // override default config file
-          // ensure there's another non-flag argument
-          if ((arg + 1) == argvec.end() || (arg + 1)[0] == L'-') {
-            Usage();
-            return false;
-          } else {
-            arg++;
-            // No need to set config file as done in previous parsing
-            // of command line to get config information
-            if (!pws_os::FileExists(std::wstring(*arg))) {
-              // FileExists returns false if specified file not found
-              CGeneralMsgBox gmb;
-              CString missing_cfg;
-              // Classic chicken-and-egg here: Since the local
-              // language could be specified in the config file, and since
-              // the desired config file can't be found, it seems
-              // safest to keep this particular error message in English
-              // and out of the language-specific dlls.
-              missing_cfg.Format(L"Configuration file %s not found - creating it.",
-                                 static_cast<LPCWSTR>(*arg));
-              gmb.AfxMessageBox(missing_cfg, L"Error", MB_OK | MB_ICONINFORMATION);
-            }
-          }
+          // These have already been processed by member function GetConfigFromCommandLine
+          arg++;
           break;
         case L'Q': case L'q': // be Quiet re missing fonts, dlls, etc.
           m_noSysEnvWarnings = true;
@@ -1014,12 +1067,15 @@ BOOL ThisMfcApp::InitInstance()
 
     if (!sxUser.empty())
       SysInfo::GetInstance()->SetEffectiveUser(sxUser.c_str());
+  } else {
+    // Issues - get out
+    return FALSE;
   }
 
   // Set up PWSprefs for this PasswordSafe session.
   // Whilst the language may be changed by the user via a menu, there is no user
   // facility to change the config file, host or user later.
-  PWSprefs::GetInstance();
+  PWSprefs *prefs = PWSprefs::GetInstance();
 
   // Now load translations (localized messages used in Usage and encrypt/decrypt dialogs)
   LoadLocalizedStuff();
@@ -1036,9 +1092,9 @@ BOOL ThisMfcApp::InitInstance()
   std::bitset<UIInterFace::NUM_SUPPORTED> bsSupportedFunctions;
   bsSupportedFunctions.set(UIInterFace::DATABASEMODIFIED);
   bsSupportedFunctions.set(UIInterFace::UPDATEGUI);
-  bsSupportedFunctions.set(UIInterFace::GUISETUPDISPLAYINFO);
   bsSupportedFunctions.set(UIInterFace::GUIREFRESHENTRY);
   bsSupportedFunctions.set(UIInterFace::UPDATEWIZARD);
+  bsSupportedFunctions.set(UIInterFace::UPDATEGUIGROUPS);
 
   m_core.SetUIInterFace(&dbox, UIInterFace::NUM_SUPPORTED, bsSupportedFunctions);
 
@@ -1055,8 +1111,6 @@ BOOL ThisMfcApp::InitInstance()
   else if (!parseVal) // bad command line args
     return FALSE;
 
-  PWSprefs *prefs = PWSprefs::GetInstance();
-
   // And the others - even if not referenced here
   Fonts *pFonts = Fonts::GetInstance();
   PWSversion *pPWSver = PWSversion::GetInstance();
@@ -1065,7 +1119,7 @@ BOOL ThisMfcApp::InitInstance()
   UNREFERENCED_PARAMETER(pFonts);
   UNREFERENCED_PARAMETER(pPWSver);
 
-  // Update Quiet value if via environmental variable rather than 
+  // Update Quiet value if via environment variable rather than 
   // command line flag
   CString cs_PWS_QUIET;
   if (cs_PWS_QUIET.GetEnvironmentVariable(L"PWS_QUIET") != FALSE)
@@ -1160,7 +1214,7 @@ BOOL ThisMfcApp::InitInstance()
   }
 
   // Run dialog - note that we don't particularly care what the response was
-  (void) dbox.DoModal();
+  dbox.DoModal();
 
   if (m_pTrayIcon != NULL)
     m_pTrayIcon->DestroyWindow();
@@ -1201,38 +1255,38 @@ void ThisMfcApp::ClearMRU()
 
   // Can't get the MRU list on the menu to tidy up automatically
   // Do it manually!
-  CWnd* pMain = AfxGetMainWnd();
+  CWnd *pMain = AfxGetMainWnd();
 
-  CMenu* xmainmenu = pMain->GetMenu();
+  CMenu *pMainMenu = pMain->GetMenu();
 
   // Look for "File" menu.
-  int pos = FindMenuItem(xmainmenu, ID_FILEMENU);
+  int pos = FindMenuItem(pMainMenu, ID_FILEMENU);
 
-  CMenu* xfile_submenu = xmainmenu->GetSubMenu(pos);
-  if (xfile_submenu != NULL)  // Look for MRU first entry
-    pos = FindMenuItem(xfile_submenu, ID_FILE_MRU_ENTRY1);
+  CMenu *pFile_Submenu = pMainMenu->GetSubMenu(pos);
+  if (pFile_Submenu != NULL)  // Look for MRU first entry
+    pos = FindMenuItem(pFile_Submenu, ID_FILE_MRU_ENTRY1);
   else
     return;
 
   if (pos > -1) {
     // Recent databases are on the main File menu - remove them
     for (int nID = numMRU; nID > 1; nID--)
-      xfile_submenu->RemoveMenu(ID_FILE_MRU_ENTRY1 + nID - 1, MF_BYCOMMAND);
+      pFile_Submenu->RemoveMenu(ID_FILE_MRU_ENTRY1 + nID - 1, MF_BYCOMMAND);
 
     return;
   }
 
   // Recent databases are on the popup menu off the main File menu
-  CMenu* xpopupmenu = xfile_submenu->GetSubMenu(3);
-  if (xpopupmenu != NULL)  // Look for MRU first entry
-    pos = FindMenuItem(xpopupmenu, ID_FILE_MRU_ENTRY1);
+  CMenu *pPopupMenu = pFile_Submenu->GetSubMenu(3);
+  if (pPopupMenu != NULL)  // Look for MRU first entry
+    pos = FindMenuItem(pPopupMenu, ID_FILE_MRU_ENTRY1);
   else
     return;
 
   if (pos > -1) {
     // Recent databases are on a popup menu - remove them
     for (int nID = numMRU; nID > 1; nID--)
-      xfile_submenu->RemoveMenu(ID_FILE_MRU_ENTRY1 + nID - 1, MF_BYCOMMAND);
+      pFile_Submenu->RemoveMenu(ID_FILE_MRU_ENTRY1 + nID - 1, MF_BYCOMMAND);
 
     return;
   }
@@ -1309,7 +1363,7 @@ BOOL ThisMfcApp::ProcessMessageFilter(int code, LPMSG lpMsg)
 // FindMenuItem() will find a menu item string from the specified
 // popup menu and returns its position (0-based) in the specified
 // popup menu. It returns -1 if no such menu item string is found.
-int ThisMfcApp::FindMenuItem(CMenu* Menu, LPCWSTR MenuString)
+int ThisMfcApp::FindMenuItem(CMenu *Menu, LPCWSTR MenuString)
 {
   ASSERT(Menu);
   ASSERT(::IsMenu(Menu->GetSafeHmenu()));
@@ -1328,7 +1382,7 @@ int ThisMfcApp::FindMenuItem(CMenu* Menu, LPCWSTR MenuString)
 // FindMenuItem() will find a menu item ID from the specified
 // popup menu and returns its position (0-based) in the specified
 // popup menu. It returns -1 if no such menu item string is found.
-int ThisMfcApp::FindMenuItem(CMenu* Menu, UINT MenuID)
+int ThisMfcApp::FindMenuItem(CMenu *Menu, UINT MenuID)
 {
   ASSERT(Menu);
   ASSERT(::IsMenu(Menu->GetSafeHmenu()));
@@ -1359,7 +1413,7 @@ void ThisMfcApp::GetApplicationVersionData()
   ::GetModuleFileName(NULL, szFullPath, sizeof(szFullPath));
   dwVerInfoSize = ::GetFileVersionInfoSize(szFullPath, &dwVerHnd);
   if (dwVerInfoSize > 0) {
-    char* pVersionInfo = new char[dwVerInfoSize];
+    char *pVersionInfo = new char[dwVerInfoSize];
     if (pVersionInfo != NULL) {
       BOOL bRet = ::GetFileVersionInfo((LPWSTR)szFullPath,
                                        (DWORD)dwVerHnd,
@@ -1386,7 +1440,7 @@ void ThisMfcApp::GetApplicationVersionData()
         CString cs_text;
         wchar_t *buffer, *lpsztext;
         UINT buflen;
-        TRANSARRAY* lpTransArray;
+        TRANSARRAY *lpTransArray;
 
         VerQueryValue(pVersionInfo, L"\\VarFileInfo\\Translation",
                      (LPVOID*)&buffer, &buflen);
@@ -1448,7 +1502,7 @@ void ThisMfcApp::GetDLLVersionData(const CString &cs_dll, int &wLangID)
   // Get version information from the DLL
   dwVerInfoSize = ::GetFileVersionInfoSize(cs_dll, &dwVerHnd);
   if (dwVerInfoSize > 0) {
-    char* pVersionInfo = new char[dwVerInfoSize];
+    char *pVersionInfo = new char[dwVerInfoSize];
     if (pVersionInfo != NULL) {
       BOOL bRet = ::GetFileVersionInfo((LPCWSTR)cs_dll,
                                        (DWORD)dwVerHnd,
@@ -1521,7 +1575,7 @@ void ThisMfcApp::GetLanguageFiles()
     //                               012345678901           012345678901234
     // length should be either 10+2 (pwsafeLL.dll) or 10+5 (pwsafeLL_CC.dll)
     int len = cs_dll.GetLength();
-    ASSERT(len == 12 || len == 15);
+
     if (len != 12 && len != 15)
       continue;
 
