@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2017 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -239,7 +239,7 @@ int CWZFinish::ExecuteAction()
 
   if (bOtherIsDB) {
     // Not really needed but...
-    m_pothercore->ClearData();
+    m_pothercore->ClearDBData();
 
     // Reading a new file changes the preferences as they are instance dependent
     // not core dependent
@@ -282,7 +282,7 @@ int CWZFinish::ExecuteAction()
     }
 
     if (rc != PWScore::SUCCESS) {
-      m_pothercore->ClearData();
+      m_pothercore->ClearDBData();
       m_pothercore->SetCurFile(L"");
       delete m_pothercore;
       m_pothercore = NULL;
@@ -394,7 +394,7 @@ LRESULT CWZFinish::OnExecuteThreadEnded(WPARAM , LPARAM )
     if (m_pothercore->IsLockedFile(m_pothercore->GetCurFile().c_str()))
       m_pothercore->UnlockFile(m_pothercore->GetCurFile().c_str());
 
-    m_pothercore->ClearData();
+    m_pothercore->ClearDBData();
     m_pothercore->SetCurFile(L"");
     delete m_pothercore;
     m_pothercore = NULL;
